package com.jln.system.web.action.interceptor;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.jln.common.constant.MesConfiguration;
import com.jln.system.web.action.utils.ApplicationContextHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.plugin.Intercepts;
import org.apache.ibatis.plugin.Invocation;
import org.apache.ibatis.plugin.Signature;
import org.apache.ibatis.reflection.DefaultReflectorFactory;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;


/**
 * <p>
 *  自定义sql关键字替换，主要是根据mysql,oracle,sqlserver的不同，替换关键字 mysql 是反单引号 oracle是双引号 sqlserver是[]
 * </p>
 *
 * @author 刘辉军
 * @since 2022.04.25
 */

@Slf4j
@Component
@Intercepts({@Signature(type = StatementHandler.class, method = "prepare", args = {Connection.class, Integer.class})})
public class SqlReplaceInterceptor implements Interceptor {
    private MesConfiguration mesConfiguration;
    private ApplicationContext beanFactory;
    /**
     * BaseMapper方法名集合
     */
    private List<String> mapperMethodNameList = new ArrayList<>();

    {
        Class<BaseMapper> clazz = BaseMapper.class;
        Method[] methods = clazz.getMethods();
        for (Method method : methods) {
            mapperMethodNameList.add(method.getName());
        }
    }

    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        // Sql拦截-环境变量过滤
        this.envSqlInterceptor(invocation);
        return invocation.proceed();
    }

    @Override
    public Object plugin(Object target) {
        return Interceptor.super.plugin(target);
    }

    @Override
    public void setProperties(Properties properties) {
        Interceptor.super.setProperties(properties);
    }

    /**
     * sql拦截-环境变量过滤 用于处理sql关键字过滤 ，mysql可以加反单引号，oracle不能，所以替换成双引号
     * 说明：
     * 对于mybaties-plus方法，需要在实体类上加EnvSqlFilter注解
     * 对于自定义方法，需要在mapper的方法上增加EnvSqlFilter注解
     *
     * @param invocation
     */
    private void envSqlInterceptor(Invocation invocation) throws Throwable {
        this.loadService();

        StatementHandler statementHandler = (StatementHandler) invocation.getTarget();
        MetaObject metaObject = MetaObject.forObject(statementHandler, SystemMetaObject.DEFAULT_OBJECT_FACTORY, SystemMetaObject.DEFAULT_OBJECT_WRAPPER_FACTORY, new DefaultReflectorFactory());

        //拼装sql
        BoundSql boundSql = statementHandler.getBoundSql();
        String oldSql = boundSql.getSql();


        String newSql = buildNewSql(oldSql);
        //设置新sql

        metaObject.setValue("delegate.boundSql.sql", newSql);
        log.debug("baseSql:{} authSql:{}", oldSql, newSql);

    }
    /**
     * 替换掉invocation中原有的MappedStatement
     *
     * @param mappedStatement 老statement
     * @param boundSqlSource  绑定sql对象
     * @return 新MappedStatement
     */


        /**
         * 构建新的sql
         * @param oldSql 原sql
         * @return 处理之后的sql
         */
        private String buildNewSql (String oldSql) {

            try {


                String databaseProductName = mesConfiguration.getDbReleaseType();
                databaseProductName = databaseProductName.toLowerCase();
                // 进行反单引号替换成双引号
                if (databaseProductName.contains("oracle")) {
                    oldSql = oldSql.replaceAll("`", "\"");
                }

//            // 1. 主表是否有别名，如果有则加上别名
//            Alias alias = selectBody.getFromItem().getAlias();
//            if (alias != null) {
//                appendWhereSql = alias.getName() + "." + appendWhereSql;
//            }
//            // 2. 如果有where语句，则追加。没有则直接作为where语句
//            String newWhereSql = appendWhereSql;
//            Expression oldWhere = selectBody.getWhere();
//            if (oldWhere != null) {
//                newWhereSql = appendWhereSql + "and (" + oldWhere + ")";
//            }
//            Expression newWhere = CCJSqlParserUtil.parseCondExpression(newWhereSql);
//            selectBody.setWhere(newWhere);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return oldSql;
        }


    private void loadService() {
        if (null == beanFactory) {
            beanFactory = ApplicationContextHelper.getApplicationContext();
            if(null == beanFactory){
                return;
            }
        }
        if(null==mesConfiguration){
            mesConfiguration = beanFactory.getBean(MesConfiguration.class);
        }
    }

}